searchData={"items":[{"type":"module","title":"Specter","doc":"Specter is a method for managing data structures and entities provided by\n`webrtc.rs`. It is intended as a low-level library with some small set of\nopinions, which can composed into more complex behaviors by higher-level\nlibraries and applications.","ref":"Specter.html"},{"type":"module","title":"Key points - Specter","doc":"Specter wraps `webrtc.rs`, which heavily utilizes async Rust. For this reason,\nmany functions cannot be automatically awaited by the caller—the NIF functions\nsend messages across channels to separate threads managed by Rust, which send\nmessages back to Elixir that can be caught by `receive` or `handle_info`.","ref":"Specter.html#module-key-points"},{"type":"module","title":"Usage - Specter","doc":"A process initializes Specter via the `init/1` function, which registers the\ncurrent process for callbacks that may be triggered via webrtc entities.\n\n    iex>","ref":"Specter.html#module-usage"},{"type":"module","title":"Initialize the library. Register messages to the current pid. - Specter","doc":"iex> {:ok, specter} = Specter.init(ice_servers: [\"stun:stun.l.google.com:19302\"])\n    ...>\n    iex>","ref":"Specter.html#module-initialize-the-library-register-messages-to-the-current-pid"},{"type":"module","title":"Create a peer connection's dependencies - Specter","doc":"iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    ...>\n    iex> Specter.media_engine_exists?(specter, media_engine)\n    true\n    iex> Specter.registry_exists?(specter, registry)\n    true\n    ...>\n    iex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n    ...>\n    iex> Specter.media_engine_exists?(specter, media_engine)\n    false\n    iex> Specter.registry_exists?(specter, registry)\n    false\n    ...>\n    iex>","ref":"Specter.html#module-create-a-peer-connection-s-dependencies"},{"type":"module","title":"Create a peer connection - Specter","doc":"iex> {:ok, pc_1} = Specter.PeerConnection.new(specter, api)\n    iex> assert_receive {:peer_connection_ready, ^pc_1}\n    iex> Specter.PeerConnection.exists?(specter, pc_1)\n    true\n    iex>","ref":"Specter.html#module-create-a-peer-connection"},{"type":"module","title":"Add a thing to be negotiated - Specter","doc":"iex> :ok = Specter.PeerConnection.create_data_channel(specter, pc_1, \"data\")\n    iex> assert_receive {:data_channel_created, ^pc_1}\n    ...>\n    iex>","ref":"Specter.html#module-add-a-thing-to-be-negotiated"},{"type":"module","title":"Create an offer - Specter","doc":"iex> :ok = Specter.PeerConnection.create_offer(specter, pc_1)\n    iex> assert_receive {:offer, ^pc_1, offer}\n    iex> :ok = Specter.PeerConnection.set_local_description(specter, pc_1, offer)\n    iex> assert_receive {:ok, ^pc_1, :set_local_description}\n    ...>\n    iex>","ref":"Specter.html#module-create-an-offer"},{"type":"module","title":"Create a second peer connection, to answer back - Specter","doc":"iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n    iex> {:ok, pc_2} = Specter.PeerConnection.new(specter, api)\n    iex> assert_receive {:peer_connection_ready, ^pc_2}\n    ...>\n    iex>","ref":"Specter.html#module-create-a-second-peer-connection-to-answer-back"},{"type":"module","title":"Begin negotiating offer/answer - Specter","doc":"iex> :ok = Specter.PeerConnection.set_remote_description(specter, pc_2, offer)\n    iex> assert_receive {:ok, ^pc_2, :set_remote_description}\n    iex> :ok = Specter.PeerConnection.create_answer(specter, pc_2)\n    iex> assert_receive {:answer, ^pc_2, answer}\n    iex> :ok = Specter.PeerConnection.set_local_description(specter, pc_2, answer)\n    iex> assert_receive {:ok, ^pc_2, :set_local_description}\n    ...>\n    iex>","ref":"Specter.html#module-begin-negotiating-offer-answer"},{"type":"module","title":"Receive ice candidates - Specter","doc":"iex> assert_receive {:ice_candidate, ^pc_1, _candidate}\n    iex> assert_receive {:ice_candidate, ^pc_2, _candidate}\n    ...>\n    iex>","ref":"Specter.html#module-receive-ice-candidates"},{"type":"module","title":"Shut everything down - Specter","doc":"iex> Specter.PeerConnection.close(specter, pc_1)\n    :ok\n    iex> assert_receive {:peer_connection_closed, ^pc_1}\n    ...>\n    iex> :ok = Specter.PeerConnection.close(specter, pc_2)\n    iex> assert_receive {:peer_connection_closed, ^pc_2}","ref":"Specter.html#module-shut-everything-down"},{"type":"module","title":"Thoughts - Specter","doc":"During development of the library, it can be assumed that callers will\nimplement `handle_info/2` function heads appropriate to the underlying\nimplementation. Once these are more solid, it would be nice to `use Specter`,\nwhich will inject a `handle_info/2` callback, and send the messages to\nother callback functions defined by a behaviour. `handle_ice_candidate`,\nand so on.\n\nSome things are returned from the NIF as UUIDs. These are declared as `@opaque`,\nto indicate that users of the library should not rely of them being in a\nparticular format. They could change later to be references, for instance.","ref":"Specter.html#module-thoughts"},{"type":"function","title":"Specter.config/1","doc":"Returns the current configuration for the initialized NIF.","ref":"Specter.html#config/1"},{"type":"function","title":"Usage - Specter.config/1","doc":"iex> {:ok, specter} = Specter.init(ice_servers: [\"stun:stun.example.com:3478\"])\n    iex> Specter.config(specter)\n    {:ok, %Specter.Config{ice_servers: [\"stun:stun.example.com:3478\"]}}","ref":"Specter.html#config/1-usage"},{"type":"function","title":"Specter.init/1","doc":"Initialize the library. This registers the calling process to receive\ncallback messages to `handle_info/2`.\n\n| param         | type               | default |\n| ------------- | ------------------ | ------- |\n| `ice_servers` | `list(String.t())` | `[\"stun:stun.l.google.com:19302\"]` |","ref":"Specter.html#init/1"},{"type":"function","title":"Usage - Specter.init/1","doc":"iex> {:ok, _specter} = Specter.init(ice_servers: [\"stun:stun.example.com:3478\"])","ref":"Specter.html#init/1-usage"},{"type":"function","title":"Specter.media_engine_exists?/2","doc":"Returns true or false, depending on whether the media engine is available for\nconsumption, i.e. is initialized and has not been used by a function that takes\nownership of it.","ref":"Specter.html#media_engine_exists?/2"},{"type":"function","title":"Usage - Specter.media_engine_exists?/2","doc":"iex> {:ok, specter} = Specter.init(ice_servers: [\"stun:stun.l.google.com:19302\"])\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> Specter.media_engine_exists?(specter, media_engine)\n    true\n\n    iex> {:ok, specter} = Specter.init(ice_servers: [\"stun:stun.l.google.com:19302\"])\n    iex> Specter.media_engine_exists?(specter, UUID.uuid4())\n    false","ref":"Specter.html#media_engine_exists?/2-usage"},{"type":"function","title":"Specter.new_api/3","doc":"An APIBuilder is used to create RTCPeerConnections. This accepts as parameters\nthe output of `init/1`, `new_media_enine/1`, and `new_registry/2`.\n\nNote that this takes ownership of both the media engine and the registry,\neffectively consuming them.\n\n| param          | type     | default |\n| -------------- | -------- | ------- |\n| `specter`      | `t()`    | |\n| `media_engine` | `opaque` | |\n| `registry`     | `opaque` | |","ref":"Specter.html#new_api/3"},{"type":"function","title":"Usage - Specter.new_api/3","doc":"iex> {:ok, specter} = Specter.init(ice_servers: [\"stun:stun.l.google.com:19302\"])\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> {:ok, _api} = Specter.new_api(specter, media_engine, registry)","ref":"Specter.html#new_api/3-usage"},{"type":"function","title":"Specter.new_media_engine/1","doc":"Creates a MediaEngine to be configured and used by later function calls.\nCodecs and other high level configuration are done on instances of MediaEngines.\nA MediaEngine is combined with a Registry in an entity called an APIBuilder,\nwhich is then used to create RTCPeerConnections.","ref":"Specter.html#new_media_engine/1"},{"type":"function","title":"Usage - Specter.new_media_engine/1","doc":"iex> {:ok, specter} = Specter.init(ice_servers: [\"stun:stun.l.google.com:19302\"])\n    iex> {:ok, _media_engine} = Specter.new_media_engine(specter)","ref":"Specter.html#new_media_engine/1-usage"},{"type":"function","title":"Specter.new_registry/2","doc":"Creates an intercepter registry. This is a user configurable RTP/RTCP pipeline,\nand provides features such as NACKs and RTCP Reports. A registry must be created for\neach peer connection.\n\nThe registry may be combined with a MediaEngine in an API (consuming both). The API\ninstance is then used to create RTCPeerConnections.\n\nNote that creating a registry does **not** take ownership of the media engine.","ref":"Specter.html#new_registry/2"},{"type":"function","title":"Usage - Specter.new_registry/2","doc":"iex> {:ok, specter} = Specter.init(ice_servers: [\"stun:stun.l.google.com:19302\"])\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, _registry} = Specter.new_registry(specter, media_engine)\n    ...>\n    iex> Specter.media_engine_exists?(specter, media_engine)\n    true","ref":"Specter.html#new_registry/2-usage"},{"type":"function","title":"Specter.registry_exists?/2","doc":"Returns true or false, depending on whether the registry is available for\nconsumption, i.e. is initialized and has not been used by a function that takes\nownership of it.","ref":"Specter.html#registry_exists?/2"},{"type":"function","title":"Usage - Specter.registry_exists?/2","doc":"iex> {:ok, specter} = Specter.init(ice_servers: [\"stun:stun.l.google.com:19302\"])\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> Specter.registry_exists?(specter, registry)\n    true\n\n    iex> {:ok, specter} = Specter.init(ice_servers: [\"stun:stun.l.google.com:19302\"])\n    iex> Specter.registry_exists?(specter, UUID.uuid4())\n    false","ref":"Specter.html#registry_exists?/2-usage"},{"type":"opaque","title":"Specter.api_t/0","doc":"`t:Specter.api_t/0` represent an instantiated API managed in the NIF.","ref":"Specter.html#t:api_t/0"},{"type":"type","title":"Specter.ice_server/0","doc":"A uri in the form `protocol:host:port`, where protocol is either\n`stun` or `turn`.\n\nDefaults to `stun:stun.l.google.com:19302`.","ref":"Specter.html#t:ice_server/0"},{"type":"type","title":"Specter.init_options/0","doc":"Options for initializing RTCPeerConnections. This is set during initialization\nof the library, and later used when creating new connections.","ref":"Specter.html#t:init_options/0"},{"type":"opaque","title":"Specter.media_engine_t/0","doc":"`t:Specter.media_engine_t/0` represents an instantiated MediaEngine managed in the NIF.","ref":"Specter.html#t:media_engine_t/0"},{"type":"opaque","title":"Specter.native_t/0","doc":"`t:native_t/0` references are returned from the NIF, and represent state held\nin Rust code.","ref":"Specter.html#t:native_t/0"},{"type":"opaque","title":"Specter.registry_t/0","doc":"`t:Specter.registry_t/0` represent an instantiated intercepter Registry managed in the NIF.","ref":"Specter.html#t:registry_t/0"},{"type":"type","title":"Specter.t/0","doc":"`t:Specter.t/0` wraps the reference returned from `init/1`. All functions interacting with\nNIF state take a `t:Specter.t/0` as their first argument.","ref":"Specter.html#t:t/0"},{"type":"module","title":"Specter.Config","doc":"A representation of configuration kept in the initialized NIF.","ref":"Specter.Config.html"},{"type":"type","title":"Specter.Config.t/0","doc":"A representation of the configuration kept by the initialized NIF.\nNote that this does not map 1:1 to actual webrtc.rs data structures, but\nis an Elixir data structure into which the NIF can encode its config.","ref":"Specter.Config.html#t:t/0"},{"type":"module","title":"Specter.PeerConnection","doc":"Represents an RTCPeerConnection managed in the NIF. A running Specter instance may\nhave 0 or more peer connections at any time.\n\nUsers of Specter might choose between different topologies based on their use cases:\na Specter might be initialized per connection, and signaling messages passed between\ndifferent instances of the NIF; a Specter may be initialized per \"room,\" and all peer\nconnections for that room created within the single NIF instance; a \"room\" may be split\nacross Erlang nodes, with tracks forwarded between the nodes.","ref":"Specter.PeerConnection.html"},{"type":"function","title":"Specter.PeerConnection.add_ice_candidate/3","doc":"Given an ICE candidate, add it to the given peer connection. Assumes trickle ICE.\nCandidates must be JSON, with the keys `candidate`, `sdp_mid`, `sdp_mline_index`, and\n`username_fragment`.","ref":"Specter.PeerConnection.html#add_ice_candidate/3"},{"type":"function","title":"Specter.PeerConnection.add_track/3","doc":"Adds track to peer connection.\n\nSends back uuid of newly created rtp sender.\nThis will send message `t:rtp_sender_msg_t/0`.","ref":"Specter.PeerConnection.html#add_track/3"},{"type":"function","title":"Usage - Specter.PeerConnection.add_track/3","doc":"iex> {:ok, specter} = Specter.init()\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n    iex> {:ok, pc} = Specter.PeerConnection.new(specter, api)\n    iex> assert_receive {:peer_connection_ready, ^pc}\n    iex> codec = %Specter.RtpCodecCapability{mime_type: \"audio\"}\n    iex> {:ok, track} = Specter.TrackLocalStaticSample.new(specter, codec, \"audio\", \"specter\")\n    iex> :ok = Specter.PeerConnection.add_track(specter, pc, track)\n    iex> assert_receive {:rtp_sender, ^pc, ^track, _rtp_sender}\n    ...>\n    iex> {:error, :invalid_track} = Specter.PeerConnection.add_track(specter, pc, \"invalid_track\")","ref":"Specter.PeerConnection.html#add_track/3-usage"},{"type":"function","title":"Specter.PeerConnection.close/2","doc":"Closes an open instance of an RTCPeerConnection.","ref":"Specter.PeerConnection.html#close/2"},{"type":"function","title":"Usage - Specter.PeerConnection.close/2","doc":"iex> {:ok, specter} = Specter.init(ice_servers: [\"stun:stun.l.google.com:19302\"])\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n    iex> {:ok, pc} = Specter.PeerConnection.new(specter, api)\n    iex> assert_receive {:peer_connection_ready, ^pc}\n    ...>\n    iex> Specter.PeerConnection.close(specter, pc)\n    :ok\n    iex> {:ok, _pc} =\n    ...>     receive do\n    ...>       {:peer_connection_closed, ^pc} -> {:ok, pc}\n    ...>     after\n    ...>       500 -> {:error, :timeout}\n    ...>     end\n    ...>\n    iex> Specter.PeerConnection.exists?(specter, pc)\n    false","ref":"Specter.PeerConnection.html#close/2-usage"},{"type":"function","title":"Specter.PeerConnection.connection_state/2","doc":"Sends back state of peer connection.\nThis will send message `t:connection_state_msg_t/0`.","ref":"Specter.PeerConnection.html#connection_state/2"},{"type":"function","title":"Specter.PeerConnection.create_answer/3","doc":"Given an RTCPeerConnection where the remote description has been assigned via\n`set_remote_description/4`, create an answer that can be passed to another connection.\n\n| param             | type                 | default |\n| ----------------- | -------------------- | ------- |\n| `specter`         | `t()`                | |\n| `peer_connection` | `opaque`             | |\n| `options`         | `answer_options_t()` | voice_activity_detection: false |","ref":"Specter.PeerConnection.html#create_answer/3"},{"type":"function","title":"Specter.PeerConnection.create_data_channel/3","doc":"Creates a data channel on an RTCPeerConnection.\n\nNote: this can be useful when attempting to generate a valid offer, but where no media\ntracks are expected to be sent or received. Callbacks from data channels have not yet\nbeen implemented.","ref":"Specter.PeerConnection.html#create_data_channel/3"},{"type":"function","title":"Specter.PeerConnection.create_offer/3","doc":"Given an RTCPeerConnection, create an offer that can be passed to another connection.\n\n| param             | type                | default |\n| ----------------- | ------------------- | ------- |\n| `specter`         | `t()`               | |\n| `peer_connection` | `opaque`            | |\n| `options`         | `offer_options_t()` | voice_activity_detection: false |\n|                   |                     | ice_restart: false |","ref":"Specter.PeerConnection.html#create_offer/3"},{"type":"function","title":"Specter.PeerConnection.current_local_description/2","doc":"Sends back the value of the current session description on a peer connection. This will\nsend back JSON representing an offer or an answer when the peer connection has had\n`set_local_description/3` called and has successfully negotiated ICE. In all other cases,\n`nil` will be sent.\n\nSee `pending_local_description/2` and `local_description/2`.","ref":"Specter.PeerConnection.html#current_local_description/2"},{"type":"function","title":"Usage - Specter.PeerConnection.current_local_description/2","doc":"iex> {:ok, specter} = Specter.init()\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n    iex> {:ok, pc} = Specter.PeerConnection.new(specter, api)\n    iex> assert_receive {:peer_connection_ready, ^pc}\n    iex> Specter.PeerConnection.current_local_description(specter, pc)\n    :ok\n    iex> assert_receive {:current_local_description, ^pc, nil}","ref":"Specter.PeerConnection.html#current_local_description/2-usage"},{"type":"function","title":"Specter.PeerConnection.current_remote_description/2","doc":"Sends back the value of the current remote session description on a peer connection. This will\nsend back JSON representing an offer or an answer when the peer connection has had\n`set_remote_description/3` called and has successfully negotiated ICE. In all other cases,\n`nil` will be sent.\n\nSee `current_remote_description/2` and `remote_description/2`.","ref":"Specter.PeerConnection.html#current_remote_description/2"},{"type":"function","title":"Usage - Specter.PeerConnection.current_remote_description/2","doc":"iex> {:ok, specter} = Specter.init()\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n    iex> {:ok, pc} = Specter.PeerConnection.new(specter, api)\n    iex> assert_receive {:peer_connection_ready, ^pc}\n    iex> Specter.PeerConnection.current_remote_description(specter, pc)\n    :ok\n    iex> assert_receive {:current_remote_description, ^pc, nil}","ref":"Specter.PeerConnection.html#current_remote_description/2-usage"},{"type":"function","title":"Specter.PeerConnection.exists?/2","doc":"Returns true or false, depending on whether the RTCPeerConnection is initialized.","ref":"Specter.PeerConnection.html#exists?/2"},{"type":"function","title":"Usage - Specter.PeerConnection.exists?/2","doc":"iex> {:ok, specter} = Specter.init(ice_servers: [\"stun:stun.l.google.com:19302\"])\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n    iex> {:ok, pc} = Specter.PeerConnection.new(specter, api)\n    iex> assert_receive {:peer_connection_ready, ^pc}\n    iex> Specter.PeerConnection.exists?(specter, pc)\n    true\n\n    iex> {:ok, specter} = Specter.init(ice_servers: [\"stun:stun.l.google.com:19302\"])\n    iex> Specter.PeerConnection.exists?(specter, UUID.uuid4())\n    false","ref":"Specter.PeerConnection.html#exists?/2-usage"},{"type":"function","title":"Specter.PeerConnection.get_stats/2","doc":"Sends back a JSON encoded string representing the current stats of a peer connection.","ref":"Specter.PeerConnection.html#get_stats/2"},{"type":"function","title":"Usage - Specter.PeerConnection.get_stats/2","doc":"iex> {:ok, specter} = Specter.init()\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n    iex> {:ok, pc} = Specter.PeerConnection.new(specter, api)\n    iex> assert_receive {:peer_connection_ready, ^pc}\n    ...>\n    iex> Specter.PeerConnection.get_stats(specter, pc)\n    :ok\n    iex> assert_receive {:stats, ^pc, json}\n    iex> {:ok, _stats} = Jason.decode(json)","ref":"Specter.PeerConnection.html#get_stats/2-usage"},{"type":"function","title":"Specter.PeerConnection.ice_connection_state/2","doc":"Sends back state of ICE connection for given peer connection.\nThis will send message `t:ice_connection_state_msg_t/0`","ref":"Specter.PeerConnection.html#ice_connection_state/2"},{"type":"function","title":"Specter.PeerConnection.ice_gathering_state/2","doc":"Sends back state of ICE gathering process.\nThis will send message `t:ice_gathering_state_t/0`.","ref":"Specter.PeerConnection.html#ice_gathering_state/2"},{"type":"function","title":"Specter.PeerConnection.local_description/2","doc":"Sends back the value of the local session description on a peer connection. This will\nsend back JSON representing an offer or an answer when the peer connection has had\n`set_local_description/3` called. If ICE has been successfully negotated, the current\nlocal description will be sent back, otherwise the caller will receive the pending\nlocal description.\n\nSee `current_local_description/2` and `pending_local_description/2`.","ref":"Specter.PeerConnection.html#local_description/2"},{"type":"function","title":"Usage - Specter.PeerConnection.local_description/2","doc":"iex> {:ok, specter} = Specter.init()\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n    iex> {:ok, pc} = Specter.PeerConnection.new(specter, api)\n    iex> assert_receive {:peer_connection_ready, ^pc}\n    ...>\n    iex> Specter.PeerConnection.local_description(specter, pc)\n    :ok\n    iex> assert_receive {:local_description, ^pc, nil}\n    ...>\n    iex> :ok = Specter.PeerConnection.create_offer(specter, pc)\n    iex> assert_receive {:offer, ^pc, offer}\n    iex> :ok = Specter.PeerConnection.set_local_description(specter, pc, offer)\n    iex> assert_receive {:ok, ^pc, :set_local_description}\n    ...>\n    iex> Specter.PeerConnection.local_description(specter, pc)\n    :ok\n    iex> assert_receive {:local_description, ^pc, ^offer}","ref":"Specter.PeerConnection.html#local_description/2-usage"},{"type":"function","title":"Specter.PeerConnection.new/2","doc":"Creates a new RTCPeerConnection, using an API reference created with `new_api/3`. The\nfunctionality wrapped by this function is async, so `:ok` is returned immediately.\nCallers should listen for the `{:peer_connection_ready, peer_connection_t()}` message\nto receive the results of this function.\n\n| param     | type     | default |\n| --------- | -------- | ------- |\n| `specter` | `t()`    | |\n| `api`     | `opaque` | |","ref":"Specter.PeerConnection.html#new/2"},{"type":"function","title":"Usage - Specter.PeerConnection.new/2","doc":"iex> {:ok, specter} = Specter.init(ice_servers: [\"stun:stun.l.google.com:19302\"])\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n    iex> {:ok, pc} = Specter.PeerConnection.new(specter, api)\n    ...>\n    iex> {:ok, _pc} =\n    ...>     receive do\n    ...>       {:peer_connection_ready, ^pc} -> {:ok, pc}\n    ...>     after\n    ...>       500 -> {:error, :timeout}\n    ...>     end","ref":"Specter.PeerConnection.html#new/2-usage"},{"type":"function","title":"Specter.PeerConnection.pending_local_description/2","doc":"Sends back the value of the session description on a peer connection that is pending\nconnection, or nil.","ref":"Specter.PeerConnection.html#pending_local_description/2"},{"type":"function","title":"Usage - Specter.PeerConnection.pending_local_description/2","doc":"iex> {:ok, specter} = Specter.init()\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n    iex> {:ok, pc} = Specter.PeerConnection.new(specter, api)\n    iex> assert_receive {:peer_connection_ready, ^pc}\n    ...>\n    iex> Specter.PeerConnection.pending_local_description(specter, pc)\n    :ok\n    iex> assert_receive {:pending_local_description, ^pc, nil}\n    ...>\n    iex> :ok = Specter.PeerConnection.create_offer(specter, pc)\n    iex> assert_receive {:offer, ^pc, offer}\n    iex> :ok = Specter.PeerConnection.set_local_description(specter, pc, offer)\n    iex> assert_receive {:ok, ^pc, :set_local_description}\n    ...>\n    iex> Specter.PeerConnection.pending_local_description(specter, pc)\n    :ok\n    iex> assert_receive {:pending_local_description, ^pc, ^offer}","ref":"Specter.PeerConnection.html#pending_local_description/2-usage"},{"type":"function","title":"Specter.PeerConnection.pending_remote_description/2","doc":"Sends back the value of the remote session description on a peer connection on a peer\nthat is pending connection, or nil.\n\nSee `current_remote_description/2` and `pending_remote_description/2`.","ref":"Specter.PeerConnection.html#pending_remote_description/2"},{"type":"function","title":"Usage - Specter.PeerConnection.pending_remote_description/2","doc":"iex> {:ok, specter} = Specter.init()\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n    iex> {:ok, pc_offer} = Specter.PeerConnection.new(specter, api)\n    iex> assert_receive {:peer_connection_ready, ^pc_offer}\n    iex> :ok = Specter.PeerConnection.create_data_channel(specter, pc_offer, \"foo\")\n    iex> assert_receive {:data_channel_created, ^pc_offer}\n    iex> :ok = Specter.PeerConnection.create_offer(specter, pc_offer)\n    iex> assert_receive {:offer, ^pc_offer, offer}\n    ...>\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n    iex> {:ok, pc_answer} = Specter.PeerConnection.new(specter, api)\n    iex> assert_receive {:peer_connection_ready, ^pc_answer}\n    ...>\n    iex> Specter.PeerConnection.pending_remote_description(specter, pc_answer)\n    :ok\n    iex> assert_receive {:pending_remote_description, ^pc_answer, nil}\n    ...>\n    iex> :ok = Specter.PeerConnection.set_remote_description(specter, pc_answer, offer)\n    iex> assert_receive {:ok, ^pc_answer, :set_remote_description}\n    ...>\n    iex> Specter.PeerConnection.pending_remote_description(specter, pc_answer)\n    :ok\n    iex> assert_receive {:pending_remote_description, ^pc_answer, ^offer}","ref":"Specter.PeerConnection.html#pending_remote_description/2-usage"},{"type":"function","title":"Specter.PeerConnection.remote_description/2","doc":"Sends back the value of the remote session description on a peer connection. This will\nsend back JSON representing an offer or an answer when the peer connection has had\n`set_remote_description/3` called. If ICE has been successfully negotated, the current\nremote description will be sent back, otherwise the caller will receive the pending\nremote description.\n\nSee `current_remote_description/2` and `remote_description/2`.","ref":"Specter.PeerConnection.html#remote_description/2"},{"type":"function","title":"Usage - Specter.PeerConnection.remote_description/2","doc":"iex> {:ok, specter} = Specter.init()\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n    iex> {:ok, pc_offer} = Specter.PeerConnection.new(specter, api)\n    iex> assert_receive {:peer_connection_ready, ^pc_offer}\n    iex> :ok = Specter.PeerConnection.create_data_channel(specter, pc_offer, \"foo\")\n    iex> assert_receive {:data_channel_created, ^pc_offer}\n    iex> :ok = Specter.PeerConnection.create_offer(specter, pc_offer)\n    iex> assert_receive {:offer, ^pc_offer, offer}\n    ...>\n    iex> {:ok, media_engine} = Specter.new_media_engine(specter)\n    iex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n    iex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n    iex> {:ok, pc_answer} = Specter.PeerConnection.new(specter, api)\n    iex> assert_receive {:peer_connection_ready, ^pc_answer}\n    ...>\n    iex> Specter.PeerConnection.remote_description(specter, pc_answer)\n    :ok\n    iex> assert_receive {:remote_description, ^pc_answer, nil}\n    ...>\n    iex> :ok = Specter.PeerConnection.set_remote_description(specter, pc_answer, offer)\n    iex> assert_receive {:ok, ^pc_answer, :set_remote_description}\n    ...>\n    iex> Specter.PeerConnection.remote_description(specter, pc_answer)\n    :ok\n    iex> assert_receive {:remote_description, ^pc_answer, ^offer}","ref":"Specter.PeerConnection.html#remote_description/2-usage"},{"type":"function","title":"Specter.PeerConnection.set_local_description/3","doc":"Given an offer or an answer session description, sets the local description on\na peer connection. The description should be in the form of JSON with the keys\n`type` and `sdp`.\n\n| param             | type                        | default |\n| ----------------- | --------------------------- | ------- |\n| `specter`         | `t:t/0`                     | |\n| `peer_connection` | `opaque`                    | |\n| `description`     | `t:session_description_t()` | |","ref":"Specter.PeerConnection.html#set_local_description/3"},{"type":"function","title":"Specter.PeerConnection.set_remote_description/3","doc":"Given an offer or an answer in the form of SDP generated by a remote party, sets\nthe remote description on a peer connection. Expects a session description in the\nform of JSON with the keys `type` and `sdp`.\n\n| param             | type                        | default |\n| ----------------- | --------------------------- | ------- |\n| `specter`         | `t:t/0`                     | |\n| `peer_connection` | `opaque`                    | |\n| `description`     | `t:session_description_t/0` | |","ref":"Specter.PeerConnection.html#set_remote_description/3"},{"type":"function","title":"Specter.PeerConnection.signaling_state/2","doc":"Sends back state of session parameters negotiation.\nThis will send message `t:signaling_state_msg_t/0`.","ref":"Specter.PeerConnection.html#signaling_state/2"},{"type":"type","title":"Specter.PeerConnection.answer_options_t/0","doc":"Options for creating a webrtc answer. Values default to false.","ref":"Specter.PeerConnection.html#t:answer_options_t/0"},{"type":"type","title":"Specter.PeerConnection.connection_state_msg_t/0","doc":"Message sent as a result of a call to `connection_state/2`.","ref":"Specter.PeerConnection.html#t:connection_state_msg_t/0"},{"type":"type","title":"Specter.PeerConnection.connection_state_t/0","doc":"Possible states of peer connection.","ref":"Specter.PeerConnection.html#t:connection_state_t/0"},{"type":"type","title":"Specter.PeerConnection.ice_candidate_t/0","doc":"An ICE candidate as JSON.","ref":"Specter.PeerConnection.html#t:ice_candidate_t/0"},{"type":"type","title":"Specter.PeerConnection.ice_connection_state_msg_t/0","doc":"Message sent as a result of a call to `ice_connection_state/2`.","ref":"Specter.PeerConnection.html#t:ice_connection_state_msg_t/0"},{"type":"type","title":"Specter.PeerConnection.ice_connection_state_t/0","doc":"Possible states of ICE connection.","ref":"Specter.PeerConnection.html#t:ice_connection_state_t/0"},{"type":"type","title":"Specter.PeerConnection.ice_gathering_state_msg_t/0","doc":"Message sent as a result of a call to `ice_gathering_state/2`.","ref":"Specter.PeerConnection.html#t:ice_gathering_state_msg_t/0"},{"type":"type","title":"Specter.PeerConnection.ice_gathering_state_t/0","doc":"Possible states of ICE gathering process.","ref":"Specter.PeerConnection.html#t:ice_gathering_state_t/0"},{"type":"type","title":"Specter.PeerConnection.offer_options_t/0","doc":"Options for creating a webrtc offer. Values default to false.","ref":"Specter.PeerConnection.html#t:offer_options_t/0"},{"type":"type","title":"Specter.PeerConnection.rtp_sender_t/0","doc":"Message sent as a result of a call to `add_track/3`.","ref":"Specter.PeerConnection.html#t:rtp_sender_t/0"},{"type":"type","title":"Specter.PeerConnection.sdp_t/0","doc":"A UTF-8 encoded string encapsulating either an offer or an answer.","ref":"Specter.PeerConnection.html#t:sdp_t/0"},{"type":"type","title":"Specter.PeerConnection.sdp_type_t/0","doc":"The type of an SDP message, either an `:offer` or an `:answer`.","ref":"Specter.PeerConnection.html#t:sdp_type_t/0"},{"type":"type","title":"Specter.PeerConnection.session_description_t/0","doc":"A UTF-8 encoded string encapsulating an Offer or an Answer in JSON. The keys are as\nfollows:\n\n| key    | type |\n| ------ | ---- |\n| `type` | `offer`, `answer` |\n| `sdp`  | `sdp_t() |","ref":"Specter.PeerConnection.html#t:session_description_t/0"},{"type":"type","title":"Specter.PeerConnection.signaling_state_msg_t/0","doc":"Message sent as a result of a call to `signaling_state/2`.","ref":"Specter.PeerConnection.html#t:signaling_state_msg_t/0"},{"type":"type","title":"Specter.PeerConnection.signaling_state_t/0","doc":"Possible states of session parameters negotiation.","ref":"Specter.PeerConnection.html#t:signaling_state_t/0"},{"type":"opaque","title":"Specter.PeerConnection.t/0","doc":"`t:Specter.PeerConnection.t/0` represents an instantiated RTCPeerConnection managed in the NIF.","ref":"Specter.PeerConnection.html#t:t/0"},{"type":"module","title":"Specter.RtpCodecCapability","doc":"A representation of webrtc.rs RTCRtpCodecCapability.","ref":"Specter.RtpCodecCapability.html"},{"type":"type","title":"Specter.RtpCodecCapability.t/0","doc":"For the meaning of specific fields refer to\nhttps://w3c.github.io/webrtc-pc/#rtcrtpcodeccapability\n\nAdditionaly, webrtc.rs allows to specify extra RTCP packet types.\nHowever, this is not implemented yet (and also not included in w3c standard).","ref":"Specter.RtpCodecCapability.html#t:t/0"},{"type":"module","title":"Specter.TrackLocalStaticSample","doc":"A representation of webrtc.rs `TrackLocalStaticSample`.\n\nIn general, a track in WebRTC represents a single audio or video\nand its main purpose is to provide user with API for\nsending and receiving media data/packets.\n\nTherefore, webrtc.rs has multiple implementations of the track depending on\nwhat user want to do.\n\nLocal tracks are outbound tracks i.e. they are used when user wants to\nsend media to the other end of a peer connection.\nUser must instantiate local track explicitly.\nAt the moment, there are two types of local track: `TrackLocalStaticSample`\nand `TrackLocalStaticRtp`.\nThe former is used when user wants RTP encapsulation to be performed under the hood.\nThe latter, when user has already prepared RTP packets.\n\nRemote tracks are inbound tracks i.e. they represent incoming media.\nUser does not create remote track explicitly.\nInstead, it announces willingness to receive track by creating a rtp transceiver\nand then, when there are some remote packets, webrtc.rs creates a new\nremote track internally and notifies user.","ref":"Specter.TrackLocalStaticSample.html"},{"type":"function","title":"Specter.TrackLocalStaticSample.new/4","doc":"Creates new TrackLocalStaticSample.","ref":"Specter.TrackLocalStaticSample.html#new/4"},{"type":"function","title":"Usage - Specter.TrackLocalStaticSample.new/4","doc":"iex> {:ok, specter} = Specter.init()\n    iex> codec = %Specter.RtpCodecCapability{mime_type: \"audio\"}\n    iex> {:ok, _track} = Specter.TrackLocalStaticSample.new(specter, codec, \"audio\", \"specter\")","ref":"Specter.TrackLocalStaticSample.html#new/4-usage"},{"type":"function","title":"Specter.TrackLocalStaticSample.play_from_file_h264/3","doc":"Reads H264 file and writes it to the track.","ref":"Specter.TrackLocalStaticSample.html#play_from_file_h264/3"},{"type":"opaque","title":"Specter.TrackLocalStaticSample.t/0","doc":"Represents an instantiated TrackLocalStaticSample stored in the NIF.","ref":"Specter.TrackLocalStaticSample.html#t:t/0"},{"type":"extras","title":"Specter","doc":"# Specter\n\n`Specter` is a wrapper for [webrtc.rs](https://webrtc.rs) as an Elixir NIF, using Rustler.\n\nThis library is a low-level interface to the data structures and entities provided by Rust, with a\nminimal set of opinions.","ref":"readme.html"},{"type":"extras","title":"Installation - Specter","doc":"```elixir\ndef deps do\n  [\n    {:specter, \"~> 0.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Checklist - Specter","doc":"- [x] `Specter.init/1` takes (opts)\n  - opts: (`ice_servers`)\n- [x] `Specter.config/1` (ref), returning `Specter.Config.t()`\n- [x] `Specter.new_media_engine/1` (ref), returning UUID\n- [x] `Specter.new_registry/2` (ref, uuid), returning UUID\n- [x] `Specter.new_api/3` (ref, uuid, uuid), returning UUID\n  - arg1: media engine uuid\n  - arg2: registry uuid\n- [x] `Specter.PeerConnection.new/2` (ref, uuid), returning UUID\n  - arg1: api builder uuid\n- [x] `Specter.PeerConnection.close/2` (ref, uuid)\n- [x] `Specter.PeerConnection.set_remote_description/3` (ref, uuid, json)\n- [x] `Specter.PeerConnection.create_offer/3` (ref, uuid, opts)\n  - opts: (`voice_activity_detection`: `bool`, `ice_restart`: `bool`)\n- [x] `Specter.PeerConnection.create_data_channel/3` (ref, uuid, label)\n- [x] `Specter.PeerConnection.create_answer/3` (ref, uuid, opts)\n  - opts: (`voice_activity_detection`: `bool`)\n- [x] `Specter.PeerConnection.set_local_description/3` (ref, uuid, json)\n- [x] `Specter.PeerConnection.current_local_description/2`\n- [x] `Specter.PeerConnection.pending_local_description/2`\n- [x] `Specter.PeerConnection.local_description/2`\n- [x] `pc.on_ice_candidate` sends candidate to callback process\n- [x] `Specter.PeerConnection.add_ice_candidate/3` (ref, uuid, string)\n- [x] `Specter.PeerConnection.current_remote_description/2`\n- [x] `Specter.PeerConnection.pending_remote_description/2`\n- [x] `Specter.PeerConnection.remote_description/2`\n- [x] `Specter.PeerConnection.ice_connection_state/2`\n- [x] `Specter.PeerConnection.ice_gathering_state/2`\n- [x] `Specter.PeerConnection.signaling_state/2`\n- [x] `Specter.PeerConnection.connection_state/2`\n- [x] `Specter.PeerConnection.get_stats/2`\n- [ ] pc state changes sent to Elixir pid\n- [ ] `pc.gathering_complete_promise` sends message to callback process\n  - might not want to impement this\n- [ ] `Specter.close` (ref, uuid)\n- [ ] RTC metrics sent to Elixir\n- [ ] `Specter.add_track` (ref, uuid, ?)\n- [ ] `Specter.remove_track` (ref, uuid, ?)","ref":"readme.html#checklist"},{"type":"extras","title":"Development - Specter","doc":"Development of `Specter` depends on Elixir, Erlang, and Rust being available in the environment.\nSuggested setup:\n\n```shell\nasdf plugin-add erlang\nasdf plugin-add elixir\nasdf plugin-add rust\n\nbin/dev/doctor\n```\n\nCI will run tests and audit the repository, but to make sure all checks locally, the following\ncommands can be run:\n\n```shell\nbin/dev/audit\nbin/dev/test\n```\n\nThe following script is encouraged to run all checks as a part of pushing commits:\n\n```shell\nbin/dev/shipit\n```","ref":"readme.html#development"},{"type":"extras","title":"References / Thank yous - Specter","doc":"This project is indebted to the following sites and references (at the very least).\n\n- https://github.com/scrogson/franz\n- https://webrtc.rs","ref":"readme.html#references-thank-yous"},{"type":"extras","title":"Lifecycle","doc":"# Lifecycle\n\n- Initialize the library. Register messages to the current pid.\n  - `Specter.init/1`\n\n```elixir\niex> {:ok, specter} = Specter.init(ice_servers: [\"stun:stun.l.google.com:19302\"])\n```\n\n- Create a peer connection's dependencies. Creating an API consumes its MediaEngine and Registry,\n  making them unavailable for future API instances.\n  - `Specter.new_media_engine/1`\n  - `Specter.new_registry/2`\n  - `Specter.new_api/3`\n  - `Specter.media_engine_exists?/2`\n  - `Specter.registry_exists?/2`\n\n```elixir\niex> {:ok, media_engine} = Specter.new_media_engine(specter)\niex> {:ok, registry} = Specter.new_registry(specter, media_engine)\n\niex> true = Specter.media_engine_exists?(specter, media_engine)\niex> true = Specter.registry_exists?(specter, registry)\n\niex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\n\niex> false = Specter.media_engine_exists?(specter, media_engine)\niex> false = Specter.registry_exists?(specter, registry)\n```\n\n- Create a peer connection\n  - `Specter.PeerConnection.new/2`\n  - `Specter.PeerConnection.exists?/2`\n\n```elixir\niex> {:ok, pc_1} = Specter.PeerConnection.new(specter, api)\niex> :ok = receive do: ({:peer_connection_ready, ^pc_1} -> :ok),\n...>    after: (100 -> {:error, :timeout})\n\niex> true = Specter.PeerConnection.exists?(specter, pc_1)\n```\n\n- Add a thing to be negotiated\n  - `Specter.PeerConnection.create_data_channel/3`\n\n```elixir\niex> :ok = Specter.PeerConnection.create_data_channel(specter, pc_1, \"data\")\niex> :ok = receive do: ({:data_channel_created, ^pc_1} -> :ok),\n...>    after: (100 -> {:error, :timeout})\n```\n\n- Create an offer\n  - `Specter.PeerConnection.create_offer/2`\n  - `Specter.PeerConnection.set_local_description/3`\n\n```elixir\niex> :ok = Specter.PeerConnection.create_offer(specter, pc_1)\niex> {:ok, offer} = receive do: ({:offer, ^pc_1, offer} -> {:ok, offer}),\n...>    after: (100 -> {:error, :timeout})\n\niex> :ok = Specter.PeerConnection.set_local_description(specter, pc_1, offer)\niex> :ok = receive do: ({:ok, ^pc_1, :set_local_description} -> :ok),\n...>    after: (100 -> {:error, :timeout})\n```\n\n- Create a second peer connection, to answer back\n\n```elixir\niex> {:ok, media_engine} = Specter.new_media_engine(specter)\niex> {:ok, registry} = Specter.new_registry(specter, media_engine)\niex> {:ok, api} = Specter.new_api(specter, media_engine, registry)\niex> {:ok, pc_2} = Specter.PeerConnection.new(specter, api)\niex> :ok = receive do: ({:peer_connection_ready, ^pc_2} -> :ok),\n...>    after: (100 -> {:error, :timeout})\n```\n\n- Begin negotiating offer/answer\n  - `Specter.PeerConnection.set_remote_description/3`\n  - `Specter.PeerConnection.create_answer/2`\n  - `Specter.PeerConnection.set_local_description/3`\n\n```elixir\n##  give the offer to the second peer connection\niex> :ok = Specter.PeerConnection.set_remote_description(specter, pc_2, offer)\niex> :ok = receive do: ({:ok, ^pc_2, :set_remote_description} -> :ok),\n...>    after: (100 -> {:error, :timeout})\n\n##  create an answer\niex> :ok = Specter.PeerConnection.create_answer(specter, pc_2)\niex> {:ok, answer} = receive do: ({:answer, ^pc_2, answer} -> {:ok, answer}),\n...>    after: (100 -> {:error, :timeout})\n\niex> :ok = Specter.PeerConnection.set_local_description(specter, pc_2, answer)\niex> :ok = receive do: ({:ok, ^pc_2, :set_local_description} -> :ok),\n...>    after: (100 -> {:error, :timeout})\n\n##  give the answer to the first peer connection\niex> :ok = Specter.PeerConnection.set_remote_description(specter, pc_1, answer)\niex> :ok = receive do: ({:ok, ^pc_1, :set_remote_description} -> :ok),\n...>    after: (100 -> {:error, :timeout})\n```\n\n- Receive ice candidates\n  - `Specter.PeerConnection.add_ice_candidate/3`\n\n```elixir\niex> {:ok, candidate} = receive do: ({:ice_candidate, ^pc_1, c} -> {:ok, c}),\n...>    after: (100 -> {:error, :timeout})\niex> :ok = Specter.PeerConnection.add_ice_candidate(specter, pc_2, candidate)\n\n## .... and so on.\n\niex> {:ok, candidate} = receive do: ({:ice_candidate, ^pc_2, c} -> {:ok, c}),\n...>    after: (100 -> {:error, :timeout})\niex>:ok = Specter.PeerConnection.add_ice_candidate(specter, pc_1, candidate)\n\n## .... and so on.\n```\n\n- Shut everything down\n  - `Specter.PeerConnection.close/2`\n\n```elixir\niex> :ok = Specter.PeerConnection.close(specter, pc_1)\niex> receive do: ({:peer_connection_closed, ^pc_1} -> :ok),\n...>    after: (100 -> {:error, :timeout})\n:ok\n\niex> :ok = Specter.PeerConnection.close(specter, pc_2)\niex> receive do: ({:peer_connection_closed, ^pc_2} -> :ok),\n...>    after: (100 -> {:error, :timeout})\n:ok\n```","ref":"lifecycle.html"},{"type":"extras","title":"README","doc":"# Internal docs\n\nInternal docs are intended for developers. They document library internals i.e. architecture, design\ndecisions, directory structure etc.\n\nDiagrams are made in [diagrams.net](https://app.diagrams.net/).","ref":"internal_readme.html"},{"type":"extras","title":"Architecture","doc":"# Architecture\n\n![architecture](./assets/architecture.drawio.png)","ref":"architecture.html"}],"content_type":"text/markdown"}